/*
 * Copyright (c) 2020, MediaTek Inc. All rights reserved.
 *
 * This software/firmware and related documentation ("MediaTek Software") are
 * protected under relevant copyright laws.
 * The information contained herein is confidential and proprietary to
 * MediaTek Inc. and/or its licensors.
 * Except as otherwise provided in the applicable licensing terms with
 * MediaTek Inc. and/or its licensors, any reproduction, modification, use or
 * disclosure of MediaTek Software, and information contained herein, in whole
 * or in part, shall be strictly prohibited.
*/
#include <asm/byteorder.h>
#include <common.h>
#include <command.h>
#include <debug_impl.h>
#include <ecinfo.h>
#include <environment.h>
#include <ext4_common.h>
#include <ext_common.h>
#include <ext4fs.h>
#include <fs.h>
#include <jsmn.h>
#include <mtk_dataindex.h>
#include <stdio.h>
#include <stdlib.h>
#include <mapmem.h>
#include <malloc.h>
#include <memalign.h>

#include <stddef.h>

#include <vsprintf.h>
#include <utility.h>

#define MAX_PAIR_NUMBER 256
#define MAX_KEY_LENGTH 128
#define MAX_VALUE_LENGTH 128
#define MAX_EC_PATH_BOOTARGS 30
#define MAX_PCODE 8
#define FILE_PATH_SIZE 128

/* MAX_PAIR_NUMBER * (MAX_KEY_LENGTH + MAX_KEY_LENGTH) */
#define MAX_EC_CONTENT 32768

#define EC_OFFSET 8
#define EC_PATH_OFFSET 28

#ifdef CONFIG_MULTICORES_PLATFORM
#include <smp/thread_info.h>
extern smp_spin_lock_t fs_spin_lock;
#endif


//char PCODE[10];
char EC_VERSION[5] = "\0";

/* EC Ready */
int EC_READY = 0;
int EX_EC_READY = 0;
int EC_DTBO_OVERLAY_FINISH = 0;

char EC_PATH[128] = "\0";
char EC_PATH_UBOOT[128] = "\0";
char PCODE[9] = "00000000";

int EC_MAP_INIT = 0;
int SOPQ_FlashPQ_INIT = 0;
int SOPQ_LDM_INIT = 0;
int SOPQ_SOPQD_INIT = 0;
int EC_VCOM_INIT = 0;
int SOPQ_DVCFG_INIT = 0;

/***************************************************************************
 *Expanding Json format 0
 *Descript your Json map struct.
 *Only support those files depoly in ext4 partition
 map_str_t EC_MAP, SOPQ_FlashPQ, SOPQ_LDM, SOPQ_SOPQD, EC_VCOM, YOUR_JSONMAP;
     *How to get absolute path?
     *console# pwd
***************************************************************************/
/* MAP struct */
map_str_t EC_MAP, SOPQ_FlashPQ, SOPQ_LDM, EC_VCOM, SOPQ_DVCFG, SOPQ_SOPQD_043, SOPQ_SOPQD_050, SOPQ_SOPQD_055, SOPQ_SOPQD_065, SOPQ_SOPQD_075, SOPQ_SOPQD_085, SOPQ_SOPQD;

/* PCode */
#define P_CODE_CONFIG_DEFAULT "00000000"

/* Partition */
#define TVCONFIGPARTITION "tvconfig"

/* EC Path*/
#define FS_VENDOR_TVCONFIG "/vendor/tvconfig"
#define EC_BASE_INFO "base.json"
#define EC_PATH_FOLER_INIT "/ec/p"
#define EC_PATH_INIT FS_VENDOR_TVCONFIG EC_PATH_FOLER_INIT
#define EC_PATH_FOLDER_DEFAULT "/ec/default/"
#define EC_VERSION_INFO "/ec/version.txt"
#define EC_INFO_DEFAULT FS_VENDOR_TVCONFIG EC_PATH_FOLDER_DEFAULT
#define EC_BASE_INFO_DEFAULT FS_VENDOR_TVCONFIG EC_PATH_FOLDER_DEFAULT EC_BASE_INFO

/* CMDLINE */
#define CMDL_EC "EC_path="
#define CMDL_EC_PATH_PCODE_CONFIG_DEFAULT CMDL_EC EC_PATH_FOLER P_CODE_CONFIG_DEFAULT

/* EC EXPANDING */
#define FS_VENDOR_DTBO_FILE_PATH "/bsp/dtbo"
#define FS_FOLDER_SUFFIX "/"
#define FS_VENDOR_DTBO_POSITION FS_VENDOR_TVCONFIG FS_VENDOR_DTBO_FILE_PATH
#define EC_DTBO_SETTINGS_PATH "/ec/dtbOverlay.json"
#define KEY_SEPARATOR ":"
#define DTBO_SUFFIX ".dtbo"
#define SEPARATOR_OFFSET 1
#define PartitionNameLength 64
#define SOPQ_FlashPQ_MapJsonFilePath "/vendor/tvconfig/bsp/chip/mt5876/pq/FlashPqData/flashpq_variation.json"
#define SOPQ_LDM_MapJsonFilePath "/vendor/tvconfig/bsp/chip/mt5876/pq/LdData/ld_variation.json"
#define SOPQ_SOPQD_MapJsonFilePath_043 "/vendor/tvconfig/bsp/chip/mt5876/pq/SonyPQData/sopq_variation_43.json"
#define SOPQ_SOPQD_MapJsonFilePath_050 "/vendor/tvconfig/bsp/chip/mt5876/pq/SonyPQData/sopq_variation_50.json"
#define SOPQ_SOPQD_MapJsonFilePath_055 "/vendor/tvconfig/bsp/chip/mt5876/pq/SonyPQData/sopq_variation_55.json"
#define SOPQ_SOPQD_MapJsonFilePath_065 "/vendor/tvconfig/bsp/chip/mt5876/pq/SonyPQData/sopq_variation_65.json"
#define SOPQ_SOPQD_MapJsonFilePath_075 "/vendor/tvconfig/bsp/chip/mt5876/pq/SonyPQData/sopq_variation_75.json"
#define SOPQ_SOPQD_MapJsonFilePath_085 "/vendor/tvconfig/bsp/chip/mt5876/pq/SonyPQData/sopq_variation_85.json"
#define EC_VCOM_CONFIG_FilePath "/mnt/vendor/config/vcom.json"
#define EC_VCOM_PERSIST_FilePath "/mnt/vendor/persist/vcom.json"
#define SOPQ_DVCFG_MapJsonFilePath "/vendor/tvconfig/bsp/common/HDR_BIN/5.0.0.0/dolby_vision_variation.json"

static void get_token_value(jsmntok_t tok, const char * jstr, char * buf) {
    strncpy(buf, jstr + tok.start, tok.end - tok.start);
    buf[tok.end - tok.start] = '\0';
}

int parse_ec_version(void){

    char *file_buf = NULL;
    char *my_buf = NULL;
    loff_t size;

    UBOOT_TRACE("EC version txt IN\n");
    my_buf = (char *) read_storage_file_to_memory(TVCONFIGPARTITION, EC_VERSION_INFO, &size);

    if (my_buf == NULL) {
        UBOOT_DEBUG("file %s not ready for read!\n", EC_VERSION_INFO);
        return -1;
    }

    file_buf = (char *)malloc(size + 1);
    memcpy(file_buf, my_buf, size);
    file_buf[size] = '\0';
    free(my_buf);

    strncpy(EC_VERSION, file_buf, MAX_VERSION_LENGTH + 1);

    UBOOT_DEBUG("EC version is %s\n", EC_VERSION);
    free(file_buf);

    return CMD_RET_SUCCESS;
}

int check_ec_filepath_if_ready(void) {

    char buf[FILE_PATH_SIZE];

    /* use a default ec path for init test */

    /* T6y ec_path and product code logic 
     * Read product code from /mnt/vendor/persit/ProductCode.json #1
     * Read product code from /mnt/vendor/config/ProductCode.json #2
     * prioty #1 > #2
     */
    int r;
    char *file_buf = NULL;
    char *my_buf = NULL;
    loff_t size;
    jsmn_parser p;
    jsmntok_t t[20480];
    char tokbuf_key[128] = "\0";
    char tokbuf_value[128] = "\0";
    jsmn_init(&p);

/*
    if (my_buf == NULL) {
        UBOOT_DEBUG("base_read_to_Map |ec file %s not ready for read!\n", ec_base_path);
        return -3;
    }

    file_buf = (char *)malloc(size + 1);
    memcpy(file_buf, my_buf, size);
    file_buf[size] = '\0';
    free(my_buf);
*/

    UBOOT_TRACE("Product Code Json IN\n");
    my_buf = (char *) read_storage_file_to_memory("persist", "/ProductCode.json", &size);

    if(my_buf == NULL || strstr(my_buf, "\"\"") != NULL){
        UBOOT_DEBUG("Error to get Product Code in /persist turn to /config\n");
        my_buf = (char *)read_storage_file_to_memory("config", "/ProductCode.json", &size);
        if(my_buf == NULL || strstr(my_buf, "\"\"") != NULL){
            UBOOT_DEBUG("Error to get Product Code in /config turn to default value\n");
            strcpy(PCODE, "00000000");
        } else {
            file_buf = (char *)malloc(size + 1);
            memcpy(file_buf, my_buf, size);
            file_buf[size] = '\0';
            free(my_buf);
            UBOOT_DEBUG("Success to get Product Code parse in /config \n");
            r = jsmn_parse(&p, file_buf, strlen(file_buf), t, sizeof(t) / sizeof(t[0]));
            if (r < 0) {
                return CMD_RET_FAILURE;
            }

            /* Assume the top-level element is an object */
            if (r < 1 || t[0].type != JSMN_OBJECT) {
                return CMD_RET_FAILURE;
            }
            /* get KEY: productcode*/
            get_token_value(t[1], file_buf, tokbuf_key);
            if (strcmp(tokbuf_key, "product_code") == 0) {
                get_token_value(t[2], file_buf, tokbuf_value);
                strncpy(PCODE, tokbuf_value, MAX_PCODE + 1);
                free(file_buf);
            } else {
                free(file_buf);
                UBOOT_DEBUG("Error: Please check ProductCode.json in /config,it doesn't match key\n");
                return CMD_RET_FAILURE;
            }
        }
    } else {
        file_buf = (char *)malloc(size + 1);
        memcpy(file_buf, my_buf, size);
        file_buf[size] = '\0';
        free(my_buf);
        UBOOT_DEBUG("Success to get Product Code parse in /persist \n");
        r = jsmn_parse(&p, file_buf, strlen(file_buf), t, sizeof(t) / sizeof(t[0]));
        ///////////////////
        if (r < 0) {
            return CMD_RET_FAILURE;
        }

        /* Assume the top-level element is an object */
        if (r < 1 || t[0].type != JSMN_OBJECT) {
            return CMD_RET_FAILURE;
        }
        ///////////////////
        /* get KEY: PCODE*/
        get_token_value(t[1], file_buf, tokbuf_key);
        if (strcmp(tokbuf_key, "product_code") == 0) {
            get_token_value(t[2], file_buf, tokbuf_value);
            strncpy(PCODE, tokbuf_value, MAX_PCODE + 1);
            free(file_buf);
        }
        else {
            free(file_buf);
            UBOOT_DEBUG("Error: Please check ProductCode.json in /persist,it doesn't match key\n");
            return CMD_RET_FAILURE;
        }
    }
    char touch_path[256] = "/ec/p";
    strcat(touch_path, PCODE);        // /ec/
    strcat(touch_path, "/");          // /ec/pPCODE/
    strcat(touch_path, EC_BASE_INFO); // /ec/PCODE/base.json

    char * file_touchbuff = NULL;
    loff_t size_touchbuff;
    file_touchbuff = (char *)read_storage_file_to_memory(TVCONFIGPARTITION, touch_path, &size_touchbuff);
    if (file_touchbuff != NULL) {
        printf("touch_path %s exists\n", touch_path);
    } else {
        printf("touch_path %s File does not exist or cannot be opened, turn to config Pcode\n", touch_path);
        my_buf = (char *)read_storage_file_to_memory("config", "/ProductCode.json", &size);
        if(my_buf == NULL || strstr(my_buf, "\"\"") != NULL){
            UBOOT_DEBUG("Error to get Product Code in /config turn to default value\n");
            strcpy(PCODE, "00000000");
        } else {
            file_buf = (char *)malloc(size + 1);
            memcpy(file_buf, my_buf, size);
            file_buf[size] = '\0';
            free(my_buf);
            UBOOT_DEBUG("Success to get Product Code parse in /config \n");
            r = jsmn_parse(&p, file_buf, strlen(file_buf), t, sizeof(t) / sizeof(t[0]));
            if (r < 0) {
                return CMD_RET_FAILURE;
            }

            /* Assume the top-level element is an object */
            if (r < 1 || t[0].type != JSMN_OBJECT) {
                return CMD_RET_FAILURE;
            }
            /* get KEY: productcode*/
            get_token_value(t[1], file_buf, tokbuf_key);
            if (strcmp(tokbuf_key, "product_code") == 0) {
                get_token_value(t[2], file_buf, tokbuf_value);
                strncpy(PCODE, tokbuf_value, MAX_PCODE + 1);
                free(file_buf);
            } else {
                free(file_buf);
                UBOOT_DEBUG("Error: Please check ProductCode.json in /config,it doesn't match key\n");
                return CMD_RET_FAILURE;
            }
        }
    }

    if (snprintf(buf, FILE_PATH_SIZE, "PCODE=%s", PCODE) < 0)
    {
        UBOOT_DEBUG("cannot generate %s from %s\n", "PCODE", PCODE);
        return CMD_RET_FAILURE;
    }
    add_bootargs("PCODE", buf, 0);

    if(strcmp(PCODE, P_CODE_CONFIG_DEFAULT) == 0) {
        strcpy(EC_PATH, EC_INFO_DEFAULT);
        printf("default pcode generate %s to %s 0 \n","EC_PATH", EC_PATH);
    } else {
        strcpy(EC_PATH, EC_PATH_INIT); // "/vendor/tvconfig/ec/p"
        strcat(EC_PATH, PCODE);
        strcat(EC_PATH, "/");
        UBOOT_DEBUG("pcode generate %s to %s 0 \n","EC_PATH", EC_PATH);
    }

    if (snprintf(buf, FILE_PATH_SIZE, "EC_path=%s", EC_PATH) < 0)
    {
        UBOOT_DEBUG("generate %s to %s\n", "EC_path", EC_PATH);
        return CMD_RET_FAILURE;
    }

    add_bootargs("EC_path", buf, 0);

    UBOOT_TRACE("Product Code Json OUT\n");

    return CMD_RET_SUCCESS;
}

int base_read_to_Map(){

    char buf[FILE_PATH_SIZE];
    //open ec base file
    char ec_base_path[128] ="\0";
    char *filebuf = NULL;
    char *my_buf = NULL;
    loff_t size;

    UBOOT_TRACE("EC base_read_to_Map IN \n");

    if(strcmp(PCODE, P_CODE_CONFIG_DEFAULT) == 0) {
        strcpy(ec_base_path, EC_PATH_FOLDER_DEFAULT);
        strncpy(EC_PATH_UBOOT, ec_base_path, strlen(ec_base_path) + 1);      //tmp store for optional ec json
        strcat(ec_base_path, EC_BASE_INFO);
    } else {
        strcat(ec_base_path, EC_PATH_FOLER_INIT); // ec_base_path = "/ec/p"
        strcat(ec_base_path, PCODE);              // ec_base_path = "/ec/p$(PCODE)"
        strcat(ec_base_path, "/");                // ec_base_path = "/ec/p$(PCODE)/"
        strncpy(EC_PATH_UBOOT, ec_base_path, strlen(ec_base_path) + 1);      //tmp store for optional ec json
        strcat(ec_base_path, EC_BASE_INFO);
    }

    my_buf = (char *) read_storage_file_to_memory(TVCONFIGPARTITION, ec_base_path, &size);

    if (my_buf == NULL) {
        UBOOT_DEBUG("base_read_to_Map |ec file path is %s not exist turn to default!\n", ec_base_path);
        memset(ec_base_path, '\0', sizeof(ec_base_path));
        memset(EC_PATH_UBOOT, '\0', sizeof(EC_PATH_UBOOT));
        strcpy(ec_base_path, EC_PATH_FOLDER_DEFAULT);
        strncpy(EC_PATH_UBOOT, ec_base_path, strlen(ec_base_path) + 1);
        strcat(ec_base_path, EC_BASE_INFO);
        strcpy(EC_PATH, EC_INFO_DEFAULT);
        if (snprintf(buf, FILE_PATH_SIZE, "EC_path=%s", EC_PATH) < 0)
        {
            UBOOT_DEBUG("generate %s to %s\n", "EC_path", EC_PATH);
            return CMD_RET_FAILURE;
        }
        add_bootargs("EC_path", buf, 0);

        strcpy(PCODE, "00000000");
        if (snprintf(buf, FILE_PATH_SIZE, "PCODE=%s", PCODE) < 0)
        {
            UBOOT_DEBUG("cannot generate %s from %s\n", "PCODE", PCODE);
            return CMD_RET_FAILURE;
        }
        add_bootargs("PCODE", buf, 0);
        // Reset Pcode as default for  /config or /persist
        my_buf = (char *) read_storage_file_to_memory(TVCONFIGPARTITION, ec_base_path, &size);
        if (my_buf == NULL) {
            UBOOT_DEBUG("base_read_to_Map |ec deafult base.json is %s not exist !\n", ec_base_path);
            return -3;
        }
    }

    filebuf = (char *)malloc(size + 1);
    memcpy(filebuf, my_buf, size);
    filebuf[size] = '\0';
    free(my_buf);

    // load file content to buffer
    // parse json string
    int r, i;
    jsmn_parser p;
    jsmntok_t t[20480];
    char tokbuf_key[64] = "\0";
    char tokbuf_value[64] = "\0";

    char *key_buf = (char *) malloc(MAX_KEY_LENGTH);

    jsmn_init(&p);
    
/*the jsmn format:
 *{\"name\":\"MTKXXXXX\",\"admin\":true,\"uid\":11}
 *   |    ||  |        |  
 *   |    ||  |        |  
 *0 12    67  9      (17)   
 *   |    |
 * start  end
 */
    r = jsmn_parse(&p, filebuf, strlen(filebuf), t, sizeof(t) / sizeof(t[0]));

    if (r < 0) {
        UBOOT_DEBUG("Failed to parse JSON: %d\n", r);
        return -3;
    }

    /* Assume the top-level element is an object */
    if (r < 1 || t[0].type != JSMN_OBJECT) {
        UBOOT_DEBUG("JSON file corrupted: %d\n", r);
        return -3;
    }

    UBOOT_DEBUG("JSON file token number get: %d\n", r);

    //UBOOT_DEBUG("JSON file token number: %.*s", t[0].end - t[0].start, &filebuf[0] + t[0].start);
    /* Json to unordered_map */
    for (i = 1; i < r; i++) {
        //UBOOT_DEBUG("processing Key \"%.*s\" \n", t[i].end - t[i].start, &filebuf[0] + t[i].start);
        get_token_value(t[i], filebuf, tokbuf_key);
        //UBOOT_DEBUG("processing Value \"%.*s\" \n", t[i + 1].end - t[i + 1].start, &filebuf[0] + t[i + 1].start);
        get_token_value(t[i + 1], filebuf, tokbuf_value);

        //UBOOT_DEBUG("processed pair \"%s\" \"%s\" do base map insert\n", tokbuf_key, tokbuf_value);

        i = i + 1;
        char *value_buf = (char *) malloc(strlen(tokbuf_value) + 1);
        strcpy(value_buf, tokbuf_value);
        strcpy(key_buf, tokbuf_key);
        map_set(&EC_MAP, key_buf, value_buf);
        //UBOOT_DEBUG("Key \"%s\" Value \"%s\" CHECKED\"%s\" Insert Success\n", tokbuf_key, tokbuf_value, *map_get(&EC_MAP, key_buf));
    }
    free(filebuf);
    free(key_buf);
    UBOOT_DEBUG("JSON file token number processed: %d\n", i);

    UBOOT_TRACE("EC base_read_to_Map OUT \n");

    return CMD_RET_SUCCESS;
}

int optional_ec_read_to_Map(char optional_ec_path[]){

    UBOOT_DEBUG("Current processing ec optional %s\n", optional_ec_path);
    //open optional ec file
    char* filebuf = NULL;
    char* my_buf = NULL;
    loff_t size;

    UBOOT_TRACE("EC optional_ec_read_to_Map IN \n");

    // load file content to buffer
    my_buf = (char *) read_storage_file_to_memory(TVCONFIGPARTITION, optional_ec_path, &size);
    if (my_buf == NULL) {
        UBOOT_DEBUG("optional_ec_read_to_Map | ec optional file %s not ready for read!\n", optional_ec_path);
        return -4;
    }

    filebuf = (char *)malloc(size + 1);
    memcpy(filebuf, my_buf, size);
    filebuf[size] = '\0';
    free(my_buf);

    //UBOOT_DEBUG("TEST optional_ec_read_to_Map| json file content\n %s\n", filebuf);

    // parse json string
    int r, i;
    jsmn_parser p;
    jsmntok_t t[20480];
    char tokbuf_key[64] = "\0";
    char tokbuf_value[64] = "\0";

    char *key_buf = (char *) malloc(MAX_KEY_LENGTH);

    jsmn_init(&p);

/*the jsmn format:
 *{\"name\":\"MTKXXXXX\",\"admin\":true,\"uid\":11}
 *   |    ||  |        |  
 *   |    ||  |        |  
 *0 12    67  9      (17)   
 *   |    |
 * start  end
 */
    r = jsmn_parse(&p, filebuf, strlen(filebuf), t, sizeof(t) / sizeof(t[0]));

    if (r < 0) {
        UBOOT_DEBUG("Failed to parse JSON: %d\n", r);
        return -4;
    }

    /* Assume the top-level element is an object */
    if (r < 1 || t[0].type != JSMN_OBJECT) {
        UBOOT_DEBUG("JSON file corrupted: %d\n", r);
        return -4;
    }
    //UBOOT_DEBUG("JSON file token number: %.*s", t[0].end - t[0].start, &filebuf[0] + t[0].start);
    /* Json to unordered_map */
    for (i = 1; i < r; i++) {
        get_token_value(t[i], filebuf, tokbuf_key);
        UBOOT_DEBUG("Processing Key \"%s\" \n", tokbuf_key);
        get_token_value(t[i + 1], filebuf, tokbuf_value);
        UBOOT_DEBUG("Processing Value \"%s\" \n", tokbuf_value);
        char *value_buf = (char *) malloc(strlen(tokbuf_value) + 1);       
        strcpy(key_buf, tokbuf_key);
        strcpy(value_buf, tokbuf_value);
        if(map_get(&EC_MAP, key_buf) == NULL) {
            UBOOT_DEBUG("Processing Key \"%s\" is not being in base map do Insert\n", tokbuf_key);
            if (map_set(&EC_MAP, key_buf, value_buf) == 0) {
                i = i + 1;
                UBOOT_DEBUG("Key %s Value %s %s Insert Success\n", key_buf, value_buf, *map_get(&EC_MAP, key_buf));
            } else {
                UBOOT_DEBUG("KEY \"%s\" VALUE \"%s\" Insert fail !!\n", key_buf, value_buf);
                i = i + 1;
            }
        } else {
            UBOOT_DEBUG("KEY \"%s\" VALUE \"%s\" \"%s\" exist in base map go next\n",
                   tokbuf_key, tokbuf_value, *map_get(&EC_MAP, key_buf));
            i = i + 1;
        }
    }
    free(filebuf);
    UBOOT_DEBUG("JSON file token number processed: %d\n", i);
    UBOOT_TRACE("EC optional_ec_read_to_Map OUT \n");
    return CMD_RET_SUCCESS;
}

int loop_all_optional_ec(){

    /* 1. open all files in EC_PATH */
    char path[128] = "\0";
    char optional_ec_path[128] = "\0";
    char optional_ec_group[128][128];
    int optional_ec_number = 0;
    int i;

#ifdef CONFIG_MULTICORES_PLATFORM
    unsigned long irq_flag = 0;
#endif

    UBOOT_TRACE("EC loop_all_optional_ec IN \n");

    for(int i=0; i<128; i++) {
        for(int j=0; j<128; j++) {
            optional_ec_group[i][j] = '\0';
        }
    }
    strncpy(path, EC_PATH_UBOOT, strlen(EC_PATH_UBOOT) + 1);
    strncpy(optional_ec_path, EC_PATH_UBOOT, strlen(EC_PATH_UBOOT) + 1);

    char device_name[STORAGE_DEVICE_BUF_SIZE] = {0};
    char storage_info[STORAGE_INDEX_BUF_SIZE] = {0};
    unsigned int device = 0;
    int ret;

    ret = sys_get_boot_device(device_name, sizeof(device_name));
    if(ret < 0)
    {
        UBOOT_DEBUG("Error: Get booting device name failure, Unknown storage device.\n");
        return 0;
    }

    ret = sys_get_storage_info(device, TVCONFIGPARTITION, storage_info);
    if(ret < 0)
    {
        UBOOT_ERROR("Error: sys_get_storage_info failure\n");
        return CMD_RET_FAILURE;
    }

#if defined(CONFIG_MULTICORES_PLATFORM)
    smp_spin_lock_save(&fs_spin_lock, irq_flag);
#endif

    if (fs_set_blk_dev(device_name, storage_info, FS_TYPE_EXT))
    {
        UBOOT_ERROR("Error: partition select failure\n");
        goto READ_FILE_FAIL;
    }

    UBOOT_DEBUG("loop_all_optional_ec| Check current ec path %s\n", path);
    // dirStream = fs_opendir(path); _no_used
    /* fs_ls(path); */
    
    struct ext2fs_node *dirnode = NULL;
    int status;
    int dirloop = 0;
    unsigned int fpos = 0;
    loff_t actread;

    status = ext4fs_find_file(path, &ext4fs_root->diropen, &dirnode, FILETYPE_DIRECTORY);
    if (status != 1) {
        UBOOT_ERROR("loop_all_optional_ec| Can not find directory. **\n");
        if (dirnode)
            ext4fs_free_node(dirnode, &ext4fs_root->diropen);
        goto READ_FILE_FAIL;
    }

    struct ext2fs_node *diro = (struct ext2fs_node *) dirnode;
    if (!diro->inode_read) {
        status = ext4fs_read_inode(diro->data, diro->ino, &diro->inode);
        if (status == 0){
            UBOOT_ERROR("ext4fs_read_inode for diro| Can not find directory.\n");
            goto READ_FILE_FAIL;
        }
    }
    /* Search the file.  */
    while (fpos < le32_to_cpu(diro->inode.size)) {
        UBOOT_DEBUG("loop_all_optional_ec| Dir Loop #%d Begin\n", dirloop);
        struct ext2_dirent dirent;
        status = ext4fs_read_file(diro, fpos,
                                   sizeof(struct ext2_dirent),
                                   (char *)&dirent, &actread);
        if (status < 0){
            UBOOT_ERROR("ext4fs_read_file status < 0| Open the directory %s fail !!!\n", path);
            goto READ_FILE_FAIL;
        }
        if (dirent.direntlen == 0) {
            UBOOT_ERROR("ext4fs_read_file dirent.direntlen == 0| Open the directory %s fail !!!\n", path);
            goto READ_FILE_FAIL;
        }
        if (dirent.namelen != 0) {
            char filename[dirent.namelen + 1];

            UBOOT_DEBUG("loop_all_optional_ec| Open dir success then get filename\n");

            status = ext4fs_read_file(diro,
                                      fpos +
                                      sizeof(struct ext2_dirent),
                                      dirent.namelen, filename,
                                      &actread);
            if (status < 0){
                UBOOT_ERROR("ext4fs_read_file| Get filename fail!!\n");
                goto READ_FILE_FAIL;
            }
            filename[dirent.namelen] = '\0';
            //filename
            UBOOT_DEBUG("Get filename \"%s\" \n", filename);
            if (strcmp(filename, ".") != 0 && strcmp(filename, "..") != 0 \
                && strcmp(filename, "base.json") != 0 && strstr(filename, ".json") != NULL){
                UBOOT_DEBUG("Current filename \"%s\" is belong to optional ec\n", filename);
                strncpy(optional_ec_path, EC_PATH_UBOOT, strlen(EC_PATH_UBOOT) + 1);
                strncat(optional_ec_path, filename, strlen(filename) + 1);
                strncpy(optional_ec_group[optional_ec_number], optional_ec_path, strlen(optional_ec_path));
                optional_ec_number = optional_ec_number + 1;
                //UBOOT_DEBUG("TEST CURRENT filename 2 %s\n", &optional_ec_group[optional_ec_number]);
            } else {
                    UBOOT_DEBUG("Current filename \"%s\" is unexpected, please check, go next loop \n", filename);
            }
        }
        UBOOT_DEBUG("loop_all_optional_ec| Dir Loop #%d End\n", dirloop);
        dirloop = dirloop + 1;
        fpos += le16_to_cpu(dirent.direntlen);
    }

    UBOOT_DEBUG("loop_all_optional_ec| Start read EC to MAP\n");

    for(i = 0; i < optional_ec_number; i++) {
        if (optional_ec_read_to_Map(optional_ec_group[i]) >= 0)
            UBOOT_DEBUG("%s Proccessed\n", optional_ec_group[i]);
        else {
            UBOOT_ERROR("Current processing %s failed! \n", optional_ec_group[i]);
            continue;
        }
    }
        

    UBOOT_DEBUG("loop_all_optional_ec| All optional EC read to MAP success!\n");
    /* 3. close dir */
    //fs_closedir(dirStream);
    ext4fs_free_node(dirnode, &ext4fs_root->diropen);

#if defined(CONFIG_MULTICORES_PLATFORM)
    smp_spin_unlock_restore(&fs_spin_lock, irq_flag);
#endif

    UBOOT_TRACE("EC loop_all_optional_ec OUT \n");

    return CMD_RET_SUCCESS;
READ_FILE_FAIL:
#if defined(CONFIG_MULTICORES_PLATFORM)
    smp_spin_unlock_restore(&fs_spin_lock, irq_flag);
#endif
    return CMD_RET_FAILURE;

}

int ExBaseReadToMap_ProcessSingleFiles(char optional_ec_path[]){

    UBOOT_DEBUG("Current processing ec optional %s\n", optional_ec_path);
    //open optional ec file
    char* filebuf = NULL;
    char* my_buf = NULL;
    loff_t size;

    UBOOT_TRACE("EC ExBaseReadToMap_ProcessSingleFiles IN \n");

    // load file content to buffer
    my_buf = (char *) read_storage_file_to_memory(TVCONFIGPARTITION, optional_ec_path, &size);
    if (my_buf == NULL) {
        UBOOT_DEBUG("ExBaseReadToMap_ProcessSingleFiles | ec optional file %s not ready for read!\n", optional_ec_path);
        return -4;
    }

    filebuf = (char *)malloc(size + 1);
    memcpy(filebuf, my_buf, size);
    filebuf[size] = '\0';
    free(my_buf);

    //UBOOT_DEBUG("TEST optional_ec_read_to_Map| json file content\n %s\n", filebuf);

    // parse json string
    int r, i;
    jsmn_parser p;
    jsmntok_t t[20480];
    char tokbuf_key[64] = "\0";
    char tokbuf_value[64] = "\0";

    char *key_buf = (char *) malloc(MAX_KEY_LENGTH);

    jsmn_init(&p);

/*the jsmn format:
 *{\"name\":\"MTKXXXXX\",\"admin\":true,\"uid\":11}
 *   |    ||  |        |  
 *   |    ||  |        |  
 *0 12    67  9      (17)   
 *   |    |
 * start  end
 */
    r = jsmn_parse(&p, filebuf, strlen(filebuf), t, sizeof(t) / sizeof(t[0]));

    if (r < 0) {
        UBOOT_DEBUG("Failed to parse JSON: %d\n", r);
        return -4;
    }

    /* Assume the top-level element is an object */
    if (r < 1 || t[0].type != JSMN_OBJECT) {
        UBOOT_DEBUG("JSON file corrupted: %d\n", r);
        return -4;
    }
    //UBOOT_DEBUG("JSON file token number: %.*s", t[0].end - t[0].start, &filebuf[0] + t[0].start);
    /* Json to unordered_map */
    for (i = 1; i < r; i++) {
        get_token_value(t[i], filebuf, tokbuf_key);
        //UBOOT_DEBUG("Processing Key \"%s\" \n", tokbuf_key);
        get_token_value(t[i + 1], filebuf, tokbuf_value);
        //UBOOT_DEBUG("Processing Value \"%s\" \n", tokbuf_value);
        char *value_buf = (char *) malloc(strlen(tokbuf_value) + 1);       
        strcpy(key_buf, tokbuf_key);
        strcpy(value_buf, tokbuf_value);
        if(map_get(&SOPQ_SOPQD, key_buf) == NULL) {
            //UBOOT_DEBUG("Processing Key \"%s\" is not being in base map do Insert\n", tokbuf_key);
            if (map_set(&SOPQ_SOPQD, key_buf, value_buf) == 0) {
                i = i + 1;
                //UBOOT_DEBUG("Key %s Value %s %s Insert Success\n", key_buf, value_buf, *map_get(&SOPQ_SOPQD, key_buf));
            } else {
                //UBOOT_DEBUG("KEY \"%s\" VALUE \"%s\" Insert fail !!\n", key_buf, value_buf);
                i = i + 1;
            }
        } else {
            //UBOOT_DEBUG("KEY \"%s\" VALUE \"%s\" \"%s\" exist in base map go next\n", tokbuf_key, tokbuf_value, *map_get(&SOPQ_SOPQD, key_buf));
            i = i + 1;
        }
    }
    free(filebuf);
    UBOOT_DEBUG("JSON file token number processed: %d\n", i);
    UBOOT_TRACE("EC ExBaseReadToMap_ProcessSingleFiles OUT \n");
    return CMD_RET_SUCCESS;
}

int ExBaseReadToMap_MultipleFiles(map_str_t *MAP, char* MultipleFilesPath){

    /* 1. open all files in EC_PATH */
    char path[128] = "\0";
    char optional_ec_path[128] = "\0";
    char optional_ec_group[128][128];
    int optional_ec_number = 0;
    int i;

    char partition[PartitionNameLength];
    const char *relpath;

#ifdef CONFIG_MULTICORES_PLATFORM
    unsigned long irq_flag = 0;
#endif

    UBOOT_TRACE("EC ExBaseReadToMap_MultipleFiles IN \n");
    //printf("ExBaseReadToMap Init JsonMap: %s\n", MultipleFilesPath);

    if (dataindex_resolve_path(partition, sizeof(partition),
                    &relpath, MultipleFilesPath) != ERR_DATAINDEX_OK) {
        UBOOT_ERROR("ExBaseReadToMap_MultipleFiles resolves fail: %s\n", MultipleFilesPath);
        return ERR_DATAINDEX_FAIL;
    }
    //printf("ExBaseReadToMap_MultipleFiles Current Json File loacate in %s Partition\n", partition);
    //printf("ExBaseReadToMap_MultipleFiles Json relative path is %s\n", relpath);

    for(int i=0; i<128; i++) {
        for(int j=0; j<128; j++) {
            optional_ec_group[i][j] = '\0';
        }
    }
    strncpy(path, relpath, strlen(relpath) + 1);
    strncpy(optional_ec_path, relpath, strlen(relpath) + 1);

    char device_name[STORAGE_DEVICE_BUF_SIZE] = {0};
    char storage_info[STORAGE_INDEX_BUF_SIZE] = {0};
    unsigned int device = 0;
    int ret;

    ret = sys_get_boot_device(device_name, sizeof(device_name));
    if(ret < 0)
    {
        UBOOT_ERROR("Error: Get booting device name failure, Unknown storage device.\n");
        return 0;
    }

    ret = sys_get_storage_info(device, TVCONFIGPARTITION, storage_info);
    if(ret < 0)
    {
        UBOOT_ERROR("Error: sys_get_storage_info failure\n");
        return CMD_RET_FAILURE;
    }

#if defined(CONFIG_MULTICORES_PLATFORM)
    smp_spin_lock_save(&fs_spin_lock, irq_flag);
#endif

    if (fs_set_blk_dev(device_name, storage_info, FS_TYPE_EXT))
    {
        UBOOT_ERROR("Error: partition select failure\n");
        goto READ_FILE_FAIL;
    }

    UBOOT_DEBUG("ExBaseReadToMap_MultipleFiles| Check current ec path %s\n", path);
    // dirStream = fs_opendir(path); _no_used
    /* fs_ls(path); */
    
    struct ext2fs_node *dirnode = NULL;
    int status;
    int dirloop = 0;
    unsigned int fpos = 0;
    loff_t actread;

    status = ext4fs_find_file(path, &ext4fs_root->diropen, &dirnode, FILETYPE_DIRECTORY);
    if (status != 1) {
        UBOOT_ERROR("ExBaseReadToMap_MultipleFiles| Can not find directory. **\n");
        if (dirnode)
            ext4fs_free_node(dirnode, &ext4fs_root->diropen);
        goto READ_FILE_FAIL;
    }

    struct ext2fs_node *diro = (struct ext2fs_node *) dirnode;
    if (!diro->inode_read) {
        status = ext4fs_read_inode(diro->data, diro->ino, &diro->inode);
        if (status == 0){
            UBOOT_ERROR("ext4fs_read_inode for diro| Can not find directory.\n");
            goto READ_FILE_FAIL;
        }
    }
    /* Search the file.  */
    while (fpos < le32_to_cpu(diro->inode.size)) {
        //UBOOT_DEBUG("ExBaseReadToMap_MultipleFiles| Dir Loop #%d Begin\n", dirloop);
        struct ext2_dirent dirent;
        status = ext4fs_read_file(diro, fpos,
                                   sizeof(struct ext2_dirent),
                                   (char *)&dirent, &actread);
        if (status < 0){
            //UBOOT_DEBUG("ext4fs_read_file status < 0| Open the directory %s fail !!!\n", path);
            goto READ_FILE_FAIL;
        }
        if (dirent.direntlen == 0) {
            //UBOOT_DEBUG("ext4fs_read_file dirent.direntlen == 0| Open the directory %s fail !!!\n", path);
            goto READ_FILE_FAIL;
        }
        if (dirent.namelen != 0) {
            char filename[dirent.namelen + 1];

            //UBOOT_DEBUG("ExBaseReadToMap_MultipleFiles| Open dir success then get filename\n");

            status = ext4fs_read_file(diro,
                                      fpos +
                                      sizeof(struct ext2_dirent),
                                      dirent.namelen, filename,
                                      &actread);
            if (status < 0){
                //UBOOT_DEBUG("ext4fs_read_file| Get filename fail!!\n");
                goto READ_FILE_FAIL;
            }
            filename[dirent.namelen] = '\0';
            //filename
            //UBOOT_DEBUG("Get filename \"%s\" \n", filename);
            if (strcmp(filename, ".") != 0 && strcmp(filename, "..") != 0 && strstr(filename, ".json") != NULL && strstr(filename, ".zip") == 0){
                //UBOOT_DEBUG("Current filename \"%s\" is belong to optional ec\n", filename);
                strncpy(optional_ec_path, relpath, strlen(relpath) + 1);
                strncat(optional_ec_path, filename, strlen(filename) + 1);
                strncpy(optional_ec_group[optional_ec_number], optional_ec_path, strlen(optional_ec_path));
                optional_ec_number = optional_ec_number + 1;
                //UBOOT_DEBUG("TEST CURRENT filename 2 %s\n", &optional_ec_group[optional_ec_number]);
            } else {
                ;//UBOOT_DEBUG("Current filename \"%s\" is unexpected, please check, go next loop \n", filename);
            }
        }
        //UBOOT_DEBUG("ExBaseReadToMap_MultipleFiles| Dir Loop #%d End\n", dirloop);
        dirloop = dirloop + 1;
        fpos += le16_to_cpu(dirent.direntlen);
    }

    UBOOT_DEBUG("ExBaseReadToMap_MultipleFiles| Start read EC to MAP\n");

    for(i = 0; i < optional_ec_number; i++) {
        if (ExBaseReadToMap_ProcessSingleFiles(optional_ec_group[i]) >= 0)
            UBOOT_DEBUG("%s Proccessed\n", optional_ec_group[i]);
        else {
            UBOOT_DEBUG("Current processing %s failed! \n", optional_ec_group[i]);
            continue;
        }
    }
        

    UBOOT_DEBUG("ExBaseReadToMap_MultipleFiles| All optional Json read to MAP success!\n");
    /* 3. close dir */
    //fs_closedir(dirStream);
    ext4fs_free_node(dirnode, &ext4fs_root->diropen);
#if defined(CONFIG_MULTICORES_PLATFORM)
    smp_spin_unlock_restore(&fs_spin_lock, irq_flag);
#endif

    UBOOT_TRACE("EC ExBaseReadToMap_MultipleFiles OUT \n");

    return CMD_RET_SUCCESS;

READ_FILE_FAIL:
#if defined(CONFIG_MULTICORES_PLATFORM)
    smp_spin_unlock_restore(&fs_spin_lock, irq_flag);
#endif
    return CMD_RET_FAILURE;
}

/********************************************************************
 *ExBaseReadToMap(&JsonMapName, JsonFileAbPath);
 *Function: insert Json file's content into a DRAM map data struct
 *          then we could use common API ExGetJsonValue to Json value.
 *********************************************************************/

int ExBaseReadToMap(map_str_t *MAP, char* JsonFileAbPath){

    //open ec base file
    char *filebuf = NULL;
    char *my_buf = NULL;
    loff_t size;

    char partition[PartitionNameLength];
    const char *relpath;
    UBOOT_TRACE("EC ExBaseReadToMap IN \n");
    //printf("ExBaseReadToMap Init JsonMap: %s\n", JsonFileAbPath);

    if (dataindex_resolve_path(partition, sizeof(partition),
                    &relpath, JsonFileAbPath) != ERR_DATAINDEX_OK) {
        UBOOT_DEBUG("ExBaseReadToMap resolves fail: %s\n", JsonFileAbPath);
        return ERR_DATAINDEX_FAIL;
    }
    //printf("ExBaseReadToMap Current Json File loacate in %s Partition\n", partition);
    //printf("ExBaseReadToMap Json relative path is %s\n", relpath);

    my_buf = (char *) read_storage_file_to_memory(partition, relpath, &size);

    if (my_buf == NULL) {
        UBOOT_DEBUG("ExBaseReadToMap Json file %s not ready for read!\n", relpath);
        free(my_buf);
        return -3;
    }
    //printf("ExBaseReadToMap size: %lld my_buf: %s\n", size, my_buf + 11000);
    filebuf = (char *)malloc(size + 1);
    memcpy(filebuf, my_buf, size);
    filebuf[size] = '\0';
    free(my_buf);

    // load file content to buffer
    // parse json string
    int r, i;
    jsmn_parser p;
    jsmntok_t t[20480];
    char tokbuf_key[64] = "\0";
    char tokbuf_value[64] = "\0";

    char *key_buf = (char *) malloc(MAX_KEY_LENGTH);

    jsmn_init(&p);
    
/*the jsmn format:
 *{\"name\":\"MTKXXXXX\",\"admin\":true,\"uid\":11}
 *   |    ||  |        |  
 *   |    ||  |        |  
 *0 12    67  9      (17)   
 *   |    |
 * start  end
 */
    r = jsmn_parse(&p, filebuf, strlen(filebuf), t, sizeof(t) / sizeof(t[0]));
    //printf("ExBaseReadToMap filebuf: %s\n", filebuf + 11000);
    if (r < 0) {
        printf("ExBaseReadToMap Failed to parse JSON: %d\n", r);
        return -3;
    }

    /* Assume the top-level element is an object */
    if (r < 1 || t[0].type != JSMN_OBJECT) {
        UBOOT_DEBUG("ExBaseReadToMap JSON file corrupted: %d\n", r);
        return -3;
    }

    UBOOT_DEBUG("ExBaseReadToMap JSON file token number get: %d\n", r);

    //UBOOT_DEBUG("JSON file token number: %.*s", t[0].end - t[0].start, &filebuf[0] + t[0].start);
    /* Json to unordered_map */
    for (i = 1; i < r; i++) {
        //UBOOT_DEBUG("ExBaseReadToMap processing Key \"%.*s\" \n", t[i].end - t[i].start, &filebuf[0] + t[i].start);
        get_token_value(t[i], filebuf, tokbuf_key);
        //UBOOT_DEBUG("ExBaseReadToMap processing Value \"%.*s\" \n", t[i + 1].end - t[i + 1].start, &filebuf[0] + t[i + 1].start);
        get_token_value(t[i + 1], filebuf, tokbuf_value);

        //UBOOT_DEBUG("ExBaseReadToMap processed pair \"%s\" \"%s\" do base map insert\n", tokbuf_key, tokbuf_value);

        i = i + 1;
        char *value_buf = (char *) malloc(strlen(tokbuf_value) + 1);
        strcpy(value_buf, tokbuf_value);
        strcpy(key_buf, tokbuf_key);
        map_set(MAP, key_buf, value_buf);
        //UBOOT_DEBUG("ExBaseReadToMap Key \"%s\" Value \"%s\" CHECKED\"%s\" Insert Success\n", tokbuf_key, tokbuf_value, *map_get(MAP, key_buf));
    }
    free(filebuf);
    free(key_buf);
    UBOOT_DEBUG("ExBaseReadToMap JSON file token number processed: %d\n", i);
    UBOOT_TRACE("EC ExBaseReadToMap OUT \n");
    return CMD_RET_SUCCESS;
}



int get_ec_value(const char* key, char* value, int length){

    //void * char * => char ** => char *a[] => **s = a => *s
    //value = (char *)malloc(length * sizeof(char **));
    //printf("TEST EC Key %s value tmp %s\n", key, *map_get(&EC_MAP, key));
    if(EC_READY == 0)
        return CMD_RET_USAGE;

    if(EC_READY == 1) {
        if (map_get(&EC_MAP, key)){
            strcpy(value, *map_get(&EC_MAP, key));
            return 0;
        } else
            return CMD_RET_USAGE;
    } else
        return CMD_RET_USAGE;

    return 0;
}

int get_ec_version(char* version, int length){

    if(EC_READY == 0)
        return CMD_RET_USAGE;

    if(EC_READY == 1)
        strncpy(version, EC_VERSION, length);
    else
        return CMD_RET_USAGE;

    return version == NULL ? CMD_RET_USAGE :0;
}

int ExGetJsonValue(char* ExJsonFunction, const char* key, char* value, int length){

    //void * char * => char ** => char *a[] => **s = a => *s
    //value = (char *)malloc(length * sizeof(char **));
    //printf("TEST EC Key %s value tmp %s\n", key, *map_get(&EC_MAP, key));
    if(EX_EC_READY == 0)
        return CMD_RET_USAGE;

    UBOOT_DEBUG("TEST EX Json map %s Key %s\n", ExJsonFunction, key);
    if(EX_EC_READY == 1) {
        if(strcmp(ExJsonFunction, "SOPQ_FlashPQ") == 0) {
            UBOOT_DEBUG("TEST EX Json Enter 1 %s %s\n", ExJsonFunction, key);
            if(SOPQ_FlashPQ_INIT == -1 || SOPQ_FlashPQ_INIT == 0) {
                UBOOT_ERROR("TEST EX Json Enter 2.1' SOPQ_FlashPQ_INIT status error %d\n", SOPQ_FlashPQ_INIT);
                return -1;
            }
            if (map_get(&SOPQ_FlashPQ, key)){
                UBOOT_DEBUG("TEST EX Json Enter 1.1 %s %s\n", ExJsonFunction, key);
                strcpy(value, *map_get(&SOPQ_FlashPQ, key));
                UBOOT_DEBUG("TEST EX Json Enter 1.2 %s %s\n", ExJsonFunction, key);
                //map_deinit(&SOPQ_FlashPQ);
                return 0;
            } else
                return CMD_RET_USAGE;
        /****************************************************************************
         * Expanding Json format 2 
         * Add ExGetJsonValue function
         * Please add your Json map struct here.
         * Obey below coding rule
        } else if(strcmp(ExJsonFunction, "YOUR_JSONMAP") == 0) {
               if (map_get(&YOUR_JSONMAP, key)) {
                   strcpy(value, *map_get(&YOUR_JSONMAP, key));
                   return 0;
               }
           } else
               return CMD_RET_USAGE;
         ****************************************************************************/
        } else if(strcmp(ExJsonFunction, "SOPQ_LDM") == 0) {
            UBOOT_DEBUG("TEST EX Json Enter 2 %s %s\n", ExJsonFunction, key);
            if(SOPQ_LDM_INIT == -1 || SOPQ_LDM_INIT == 0) {
                UBOOT_ERROR("TEST EX Json Enter 2.1' SOPQ_LDM_INIT status error %d\n", SOPQ_LDM_INIT);
                return -1;
            }
            if (map_get(&SOPQ_LDM, key)){
                UBOOT_DEBUG("TEST EX Json Enter 2.1 %s %s\n", ExJsonFunction, key);
                strcpy(value, *map_get(&SOPQ_LDM, key));
                UBOOT_DEBUG("TEST EX Json Enter 2.2 %s %s\n", ExJsonFunction, key);
                //map_deinit(&SOPQ_LDM);
                return 0;
            } else
                return CMD_RET_USAGE;
        } else if(strcmp(ExJsonFunction, "SOPQ_SOPQD_043") == 0) {
            UBOOT_DEBUG("TEST EX Json Enter 3 %s %s\n", ExJsonFunction, key);
            if(SOPQ_SOPQD_INIT == -1 || SOPQ_SOPQD_INIT == 0) {
                UBOOT_ERROR("TEST EX Json Enter 3.1' SOPQ_SOPQD_INIT status error %d\n", SOPQ_SOPQD_INIT);
                return -1;
            }
            if (map_get(&SOPQ_SOPQD_043, key)){
                UBOOT_DEBUG("TEST EX Json Enter 3.1 %s %s\n", ExJsonFunction, key);
                strcpy(value, *map_get(&SOPQ_SOPQD_043, key));
                UBOOT_DEBUG("TEST EX Json Enter 3.2 %s %s\n", ExJsonFunction, key);
                //map_deinit(&SOPQ_SOPQD);
                return 0;
            } else
                return CMD_RET_USAGE;
        } else if(strcmp(ExJsonFunction, "EC_VCOM") == 0) {
            UBOOT_DEBUG("TEST EX Json Enter 4 %s %s\n", ExJsonFunction, key);
            if(EC_VCOM_INIT == -1 || EC_VCOM_INIT == 0) {
                UBOOT_ERROR("TEST EX Json Enter 4.1' EC_VCOM_INIT status error %d\n", EC_VCOM_INIT);
                return -1;
            }
            if (map_get(&EC_VCOM, key)){
                UBOOT_DEBUG("TEST EX Json Enter 4.1 %s %s\n", ExJsonFunction, key);
                strcpy(value, *map_get(&EC_VCOM, key));
                UBOOT_DEBUG("TEST EX Json Enter 4.2 %s %s\n", ExJsonFunction, key);
                return 0;
            } else
                return CMD_RET_USAGE;
        } else if(strcmp(ExJsonFunction, "SOPQ_DVCFG") == 0) {
            UBOOT_DEBUG("TEST EX Json Enter 5 %s %s\n", ExJsonFunction, key);
            if(SOPQ_DVCFG_INIT == -1 || SOPQ_DVCFG_INIT == 0) {
                UBOOT_ERROR("TEST EX Json Enter 5.1' SOPQ_DVCFG_INIT status error %d\n", SOPQ_DVCFG_INIT);
                return -1;
            }
            if (map_get(&SOPQ_DVCFG, key)){
                UBOOT_DEBUG("TEST EX Json Enter 5.1 %s %s\n", ExJsonFunction, key);
                strcpy(value, *map_get(&SOPQ_DVCFG, key));
                UBOOT_DEBUG("TEST EX Json Enter 5.2 %s %s\n", ExJsonFunction, key);
                //map_deinit(&SOPQ_DVCFG);
                return 0;
            } else
                return CMD_RET_USAGE;
        } else if(strcmp(ExJsonFunction, "SOPQ_SOPQD_050") == 0) {
            UBOOT_DEBUG("TEST EX Json Enter 3 %s %s\n", ExJsonFunction, key);
            if(SOPQ_SOPQD_INIT == -1 || SOPQ_SOPQD_INIT == 0) {
                UBOOT_ERROR("TEST EX Json Enter 3.1' SOPQ_SOPQD_INIT status error %d\n", SOPQ_SOPQD_INIT);
                return -1;
            }
            if (map_get(&SOPQ_SOPQD_050, key)){
                UBOOT_DEBUG("TEST EX Json Enter 3.1 %s %s\n", ExJsonFunction, key);
                strcpy(value, *map_get(&SOPQ_SOPQD_050, key));
                UBOOT_DEBUG("TEST EX Json Enter 3.2 %s %s\n", ExJsonFunction, key);
                //map_deinit(&SOPQ_SOPQD);
                return 0;
            } else
                return CMD_RET_USAGE;
        } else if(strcmp(ExJsonFunction, "SOPQ_SOPQD_055") == 0) {
            UBOOT_DEBUG("TEST EX Json Enter 3 %s %s\n", ExJsonFunction, key);
            if(SOPQ_SOPQD_INIT == -1 || SOPQ_SOPQD_INIT == 0) {
                UBOOT_ERROR("TEST EX Json Enter 3.1' SOPQ_SOPQD_INIT status error %d\n", SOPQ_SOPQD_INIT);
                return -1;
            }
            if (map_get(&SOPQ_SOPQD_055, key)){
                UBOOT_DEBUG("TEST EX Json Enter 3.1 %s %s\n", ExJsonFunction, key);
                strcpy(value, *map_get(&SOPQ_SOPQD_055, key));
                UBOOT_DEBUG("TEST EX Json Enter 3.2 %s %s\n", ExJsonFunction, key);
                //map_deinit(&SOPQ_SOPQD);
                return 0;
            } else
                return CMD_RET_USAGE;
        } else if(strcmp(ExJsonFunction, "SOPQ_SOPQD_065") == 0) {
            UBOOT_DEBUG("TEST EX Json Enter 3 %s %s\n", ExJsonFunction, key);
            if(SOPQ_SOPQD_INIT == -1 || SOPQ_SOPQD_INIT == 0) {
                UBOOT_ERROR("TEST EX Json Enter 3.1' SOPQ_SOPQD_INIT status error %d\n", SOPQ_SOPQD_INIT);
                return -1;
            }
            if (map_get(&SOPQ_SOPQD_065, key)){
                UBOOT_DEBUG("TEST EX Json Enter 3.1 %s %s\n", ExJsonFunction, key);
                strcpy(value, *map_get(&SOPQ_SOPQD_065, key));
                UBOOT_DEBUG("TEST EX Json Enter 3.2 %s %s\n", ExJsonFunction, key);
                //map_deinit(&SOPQ_SOPQD);
                return 0;
            } else
                return CMD_RET_USAGE;
        } else if(strcmp(ExJsonFunction, "SOPQ_SOPQD_075") == 0) {
            UBOOT_DEBUG("TEST EX Json Enter 3 %s %s\n", ExJsonFunction, key);
            if(SOPQ_SOPQD_INIT == -1 || SOPQ_SOPQD_INIT == 0) {
                UBOOT_ERROR("TEST EX Json Enter 3.1' SOPQ_SOPQD_INIT status error %d\n", SOPQ_SOPQD_INIT);
                return -1;
            }
            if (map_get(&SOPQ_SOPQD_075, key)){
                UBOOT_DEBUG("TEST EX Json Enter 3.1 %s %s\n", ExJsonFunction, key);
                strcpy(value, *map_get(&SOPQ_SOPQD_075, key));
                UBOOT_DEBUG("TEST EX Json Enter 3.2 %s %s\n", ExJsonFunction, key);
                //map_deinit(&SOPQ_SOPQD);
                return 0;
            } else
                return CMD_RET_USAGE;
        } else if(strcmp(ExJsonFunction, "SOPQ_SOPQD_085") == 0) {
            UBOOT_DEBUG("TEST EX Json Enter 3 %s %s\n", ExJsonFunction, key);
            if(SOPQ_SOPQD_INIT == -1 || SOPQ_SOPQD_INIT == 0) {
                UBOOT_ERROR("TEST EX Json Enter 3.1' SOPQ_SOPQD_INIT status error %d\n", SOPQ_SOPQD_INIT);
                return -1;
            }
            if (map_get(&SOPQ_SOPQD_085, key)){
                UBOOT_DEBUG("TEST EX Json Enter 3.1 %s %s\n", ExJsonFunction, key);
                strcpy(value, *map_get(&SOPQ_SOPQD_085, key));
                UBOOT_DEBUG("TEST EX Json Enter 3.2 %s %s\n", ExJsonFunction, key);
                //map_deinit(&SOPQ_SOPQD);
                return 0;
            } else
                return CMD_RET_USAGE;
        }
    } else
        return CMD_RET_USAGE;

    return 0;
}

/* TBD No Use
int ExGetJsonVersion(char * ExJsonFunction, char* version, int length){

    if(EC_READY == 0)
        return CMD_RET_USAGE;

    if(EC_READY == 1)
        strncpy(version, EC_VERSION, length);
    else
        return CMD_RET_USAGE;

    return version == NULL ? CMD_RET_USAGE :0;
}
*/

/*subec_mapresult(char * subec, char Result[][], int n)
 * 
 *Aim at those ec Uboot expanding function needing an extra json map
 *table to get the corresponding ec config,this function has three
 *parameter below.
 *
 *char * subec : name of subec json
 *char Result[][] : string stores map success Result[n][string]
 *int n : number of success map result
 *
 *all subec should deploy in /vendor/tvconfig/ec/
 *Only after ec init succeed, we could call subec_mapresult.
 *
 */
#define EC_ROOT_PATH "/ec/"
#define JSON_SUFFIX ".json"
#define SUBEC_BOOTCONFIG "bootconfig"

int subec_map_result(char * SubEcFunction, char Result[][128], int * NumECMap){

    char filepath[FILE_PATH_SIZE];
    char *filebuf = NULL;
    char *my_buf = NULL;
    loff_t size;
    int r, i;
    jsmn_parser p;
    jsmntok_t t[20480];
    char tokbuf_key[128] = "\0";
    char tokbuf_value[128] = "\0";
    char separate_key[64] = "\0";
    char separate_value[64] = "\0";
    char *separator_p = NULL;
    jsmn_init(&p);
    char * ec_value = (char *) malloc(MAX_VALUE_LENGTH + 1);
    char subec_path[FILE_PATH_SIZE];

    *NumECMap = 0;
    UBOOT_TRACE("subec_map_result IN %s\n", SubEcFunction);

    strcpy(subec_path, EC_ROOT_PATH);
    strcat(subec_path, SubEcFunction);
    strcat(subec_path, JSON_SUFFIX);

    my_buf = (char *) read_storage_file_to_memory(TVCONFIGPARTITION, subec_path, &size);
    UBOOT_TRACE("subec_map_result IN 2%s\n", SubEcFunction);
    if (my_buf == NULL) {
        UBOOT_DEBUG("file %s not ready for read!\n", subec_path);
        return -1;
    }

    filebuf = (char *)malloc(size + 1);
    memcpy(filebuf, my_buf, size);
    filebuf[size] = '\0';
    free(my_buf);
    UBOOT_DEBUG("Success to get sub ec json\n");

    r = jsmn_parse(&p, filebuf, strlen(filebuf), t, sizeof(t) / sizeof(t[0]));
    if (r < 0) {
        return CMD_RET_FAILURE;
    }

    /* Assume the top-level element is an object */
    if (r < 1 || t[0].type != JSMN_OBJECT) {
        return CMD_RET_FAILURE;
    }

    /* subec.json format 
     * "Key1:Value1" :"TargetConfig"
     */

    /* read EC and loop: 
     * get corresponding config in mapping ecsub json
     */

    /*the jsmn format:
     *{\"name\":\"MTKXXXXX\",\"admin\":true,\"uid\":11}
     *   |    ||  |        |
     *   |    ||  |        |
     *0 12    67  9      (17)
     *   |    |
     * start  end
     */
    UBOOT_DEBUG("Sub EC JSON file token number get: %d\n", r);
    UBOOT_DEBUG("Loop start to process sub ec, format \"Key:Value\" : \"ValueX\"\n");

    /* Json to map DTBO file*/
    for (i = 1; i < r; i++) {
        memset(separate_key, '\0', sizeof(separate_key));
        memset(separate_value, '\0', sizeof(separate_value));
        memset(ec_value, '\0', sizeof(ec_value));
        memset(filepath, '\0', sizeof(filepath));    
        /* Separate Key:Value of dtbOverlay.json with ':'*/
        //UBOOT_DEBUG("Sub EC Get \"Key:Value\" \"%.*s\" \n", t[i].end - t[i].start, &filebuf[0] + t[i].start);
        get_token_value(t[i], filebuf, tokbuf_key);
        separator_p = strstr(tokbuf_key, KEY_SEPARATOR);
        strncpy(separate_key, tokbuf_key, (separator_p - tokbuf_key));
        strcpy(separate_value, separator_p + SEPARATOR_OFFSET);
        UBOOT_DEBUG("Sub EC get separated key value pair \"%s\" \"%s\"\n", separate_key, separate_value);
        UBOOT_DEBUG("Sub EC processing Separated Pair ask EC use key \"%s\"\n", separate_key);
        if (get_ec_value(separate_key, ec_value, MAX_VALUE_LENGTH) < 0)
        {
            //UBOOT_DEBUG("Sub EC processing Separated Pair cannot read %s from ec, \"%s\" skip\n", separate_key, ec_value);
            i = i + 1;
            continue;
        } else {
            //UBOOT_DEBUG("Sub EC processing Separated Pair ACK ec_value \"%s\"\n", ec_value);
            if ((strcmp(separate_value, ec_value) != 0) && (strcmp(separate_value, SUBEC_BOOTCONFIG)) != 0){
                //UBOOT_DEBUG("Sub EC processing Separated Pair current separated value does not exist in ec, compare %s to %s skip\n", separate_value, ec_value);
                i = i + 1;
                continue;
            } else {
                //UBOOT_DEBUG("Sub EC processing Separated Pair current separated value map success in ec, compare %s to %s\n", separate_value, ec_value);
                get_token_value(t[i + 1], filebuf, tokbuf_value);
                strcpy(Result[*NumECMap], separate_key);    //separate_key
                strcat(Result[*NumECMap], KEY_SEPARATOR);   //separate_key:
                strcat(Result[*NumECMap], tokbuf_value);    //separate_key:tokbuf_value = EC_KEY:SUBEC_VALUE
                *NumECMap = *NumECMap + 1;
            }
        }
        i = i + 1;
    }

    UBOOT_DEBUG("Sub EC map exit\n");
    free(filebuf);
    free(ec_value);
    UBOOT_DEBUG("Sub EC JSON file token number processed: %d\n", i);
    UBOOT_TRACE("subec_map_result OUT \n");
    
    return CMD_RET_SUCCESS;
}

int ec_dtbo_overlay_pre(void)
{
    if(EC_DTBO_OVERLAY_FINISH == 1) {
        printf("EC DTBO Overlay init Finished\n");
        return CMD_RET_SUCCESS;
    }

    int ret = -ERR_DATAINDEX_OK;
    char filepath[FILE_PATH_SIZE];
    char *dtbo_str = NULL;
    char *filebuf = NULL;
    char *my_buf = NULL;
    loff_t size;
    int r, i;
    jsmn_parser p;
    jsmntok_t t[20480];
    char tokbuf_key[128] = "\0";
    char tokbuf_value[128] = "\0";
    char separate_key[64] = "\0";
    char separate_value[64] = "\0";
    char *separator_p = NULL;
    jsmn_init(&p);
    char * ec_value = (char *) malloc(MAX_VALUE_LENGTH + 1);

    UBOOT_TRACE("PRE EC DTS Overlay IN \n");
    dtbo_str = env_get(ENV_DATAINDEX_DTBO);
    if (dtbo_str && (dtbo_str[0] == '0' || dtbo_str[0] == 'n'))
    {
        printf("PRE detect env %s=%s, skip dataindex dtbo overlay\n",
                ENV_DATAINDEX_DTBO, dtbo_str);
        return ret;
    }
    free(dtbo_str);
    UBOOT_TRACE("PRE EC DTBO Overlay json IN\n");
    my_buf = (char *) read_storage_file_to_memory(TVCONFIGPARTITION, EC_DTBO_SETTINGS_PATH, &size);

    if (my_buf == NULL) {
        UBOOT_DEBUG("PRE file %s not ready for read!\n", EC_DTBO_SETTINGS_PATH);
        return -1;
    }

    filebuf = (char *)malloc(size + 1);
    memcpy(filebuf, my_buf, size);
    filebuf[size] = '\0';
    free(my_buf);
    UBOOT_DEBUG("PRE Success to get DTBO Overlay json\n");
    r = jsmn_parse(&p, filebuf, strlen(filebuf), t, sizeof(t) / sizeof(t[0]));
    if (r < 0) {
        return CMD_RET_FAILURE;
    }

    /* Assume the top-level element is an object */
    if (r < 1 || t[0].type != JSMN_OBJECT) {
        return CMD_RET_FAILURE;
    }

    /* dtbOverlay.json format 
     * "Key1:Value1" :"TargetDTBO_Name"
     */

    /* read EC and loop: 
     * get corresponding dtbo file in mapping dtbOverlay
     */

    /*the jsmn format:
     *{\"name\":\"MTKXXXXX\",\"admin\":true,\"uid\":11}
     *   |    ||  |        |
     *   |    ||  |        |
     *0 12    67  9      (17)
     *   |    |
     * start  end
     */
    UBOOT_DEBUG("PRE DTBO Overlay JSON file token number get: %d\n", r);
    UBOOT_DEBUG("PRE Loop start to process DTBO Overlay format \"Key:Value\" : \"ValueX\"\n");

    /* Json to map DTBO file*/
    for (i = 1; i < r; i++) {
        memset(separate_key, '\0', sizeof(separate_key));
        memset(separate_value, '\0', sizeof(separate_value));
        memset(ec_value, '\0', sizeof(ec_value));
        memset(filepath, '\0', sizeof(filepath));
        /* Separate Key:Value of dtbOverlay.json with ':'*/
        //UBOOT_DEBUG("PRE EC DTBO Overlay get \"Key:Value\" \"%.*s\" \n", t[i].end - t[i].start, &filebuf[0] + t[i].start);
        get_token_value(t[i], filebuf, tokbuf_key);
        separator_p = strstr(tokbuf_key, KEY_SEPARATOR);
        strncpy(separate_key, tokbuf_key, (separator_p - tokbuf_key));
        strcpy(separate_value, separator_p + SEPARATOR_OFFSET);
        //UBOOT_DEBUG("PRE EC DTBO get separated key value pair \"%s\" \"%s\"\n", separate_key, separate_value);
        //UBOOT_DEBUG("PRE EC DTBO processing Separated Pair ask EC use key \"%s\"\n", separate_key);
        if (get_ec_value(separate_key, ec_value, MAX_VALUE_LENGTH) < 0)
        {
            //UBOOT_DEBUG("PRE EC DTBO processing Separated Pair cannot read %s from ec, \"%s\" skip\n", separate_key, ec_value);
            i = i + 1;
            continue;
        } else {
            //UBOOT_DEBUG("PRE EC DTBO processing Separated Pair ACK ec_value \"%s\"\n", ec_value);
            if (strcmp(separate_value, ec_value) != 0){
                //UBOOT_DEBUG("PRE EC DTBO processing Separated Pair current separated value does not exist in ec, compare %s to %s skip\n", separate_value, ec_value);
                i = i + 1;
                continue;
            } else {
                //UBOOT_DEBUG("PRE EC DTBO processing Separated Pair current separated value map success in ec, compare %s to %s\n", separate_value, ec_value);
                get_token_value(t[i + 1], filebuf, tokbuf_value);
                //UBOOT_DEBUG("PRE EC DTBO get overlay file name %s.dtbo\n", tokbuf_value);
                //UBOOT_DEBUG("PRE EC DTBO do %s.dtbo overlay\n", tokbuf_value);
                strcpy(filepath, FS_VENDOR_DTBO_POSITION); // filepath=/tvconfig/bsp/dtbo
                strcat(filepath, FS_FOLDER_SUFFIX);       // filepath=/tvconfig/bsp/dtbo/
                strcat(filepath, separate_key);            //filepath=/tvconfig/bsp/dtbo/separate_key
                strcat(filepath, FS_FOLDER_SUFFIX);       //filepath=/tvconfig/bsp/dtbo/separate_key/
                strcat(filepath, tokbuf_value);
                strcat(filepath, DTBO_SUFFIX);
                char *key_buf_dtbo = (char *) malloc(MAX_KEY_LENGTH);
                char *value_buf_dtbo = (char *) malloc(MAX_KEY_LENGTH);
                strcpy(key_buf_dtbo, separate_key);
                strcat(key_buf_dtbo, "_dtbo");
                strcpy(value_buf_dtbo, tokbuf_value);
                if(map_get(&EC_MAP, key_buf_dtbo) == NULL) {
                    map_set(&EC_MAP, key_buf_dtbo, value_buf_dtbo);
                    printf("PRE EC DTBO overlay Record dtbo file: %s : %s success\n", key_buf_dtbo, value_buf_dtbo);
                }
                //printf("EC DTBO overlay dtbo file: %s success\n", filepath);
            }
        }
        i = i + 1;
    }
    //T6y LDM temp fixed filepath=/vendor/ld_variation/ldm_AAA_0043.dtbo
    if(map_get(&EC_MAP, "ld_variation_dtbo") == NULL) {
        map_set(&EC_MAP, "ld_variation_dtbo", "ldm_AAA_0043");
        printf("PRE EC DTBO overlay Record dtbo file: %s : %s success\n", "ld_variation_dtbo", "ldm_AAA_0043");
    }
    //printf("EC DTBO overlay dtbo file: %s success\n", filepath);

    UBOOT_DEBUG("PRE EC DTBO overlay dtbo file exit\n");
    free(filebuf);
    free(ec_value);
    UBOOT_DEBUG("PRE EC DTBO JSON file token number processed: %d\n", i);
    
    EC_DTBO_OVERLAY_FINISH = 1;
    UBOOT_TRACE("PRE EC DTS Overlay OUT \n");

    return CMD_RET_SUCCESS;

}

/**
 * ec_dtbo_overlay - overlay dtbo read from ec file
 *
 * @ft_addr[in]: the kernel dtb buffer
 * @mode[in]: the overlay mode. 'u' for uboot and 'k' for kernel
 * In fact, every time dataindex_dtbo_overlay was called, ec_dtbo_overlay
 * will be called after that to be the 3rd overlay.
 * Besides, as spec require, dataindex_dtbo_overlay will do the default DTS
 * overlay, if we set DTS overlay in ec files, ec_dtbo_overlay will do the final
 * DTS overlay.
 * Returns 0 if successful,
 * negative if an error occurs
 */

int ec_dtbo_overlay(char *ft_addr, char mode)
{
    int ret = -ERR_DATAINDEX_OK;
    char filepath[FILE_PATH_SIZE];
    char *dtbo_str = NULL;
    char *filebuf = NULL;
    char *my_buf = NULL;
    loff_t size;
    int r, i;
    jsmn_parser p;
    jsmntok_t t[20480];
    char tokbuf_key[128] = "\0";
    char tokbuf_value[128] = "\0";
    char separate_key[64] = "\0";
    char separate_value[64] = "\0";
    char *separator_p = NULL;
    jsmn_init(&p);
    char * ec_value = (char *) malloc(MAX_VALUE_LENGTH + 1);

    UBOOT_TRACE("EC DTS Overlay IN \n");
    dtbo_str = env_get(ENV_DATAINDEX_DTBO);
    if (dtbo_str && (dtbo_str[0] == '0' || dtbo_str[0] == 'n'))
    {
        printf("detect env %s=%s, skip dataindex dtbo overlay\n",
                ENV_DATAINDEX_DTBO, dtbo_str);
        return ret;
    }
    free(dtbo_str);
    UBOOT_TRACE("EC DTBO Overlay json IN\n");
    my_buf = (char *) read_storage_file_to_memory(TVCONFIGPARTITION, EC_DTBO_SETTINGS_PATH, &size);

    if (my_buf == NULL) {
        UBOOT_DEBUG("file %s not ready for read!\n", EC_DTBO_SETTINGS_PATH);
        return -1;
    }

    filebuf = (char *)malloc(size + 1);
    memcpy(filebuf, my_buf, size);
    filebuf[size] = '\0';
    free(my_buf);
    UBOOT_DEBUG("Success to get DTBO Overlay json\n");
    r = jsmn_parse(&p, filebuf, strlen(filebuf), t, sizeof(t) / sizeof(t[0]));
    if (r < 0) {
        return CMD_RET_FAILURE;
    }

    /* Assume the top-level element is an object */
    if (r < 1 || t[0].type != JSMN_OBJECT) {
        return CMD_RET_FAILURE;
    }

    /* dtbOverlay.json format 
     * "Key1:Value1" :"TargetDTBO_Name"
     */

    /* read EC and loop: 
     * get corresponding dtbo file in mapping dtbOverlay
     */

    /*the jsmn format:
     *{\"name\":\"MTKXXXXX\",\"admin\":true,\"uid\":11}
     *   |    ||  |        |
     *   |    ||  |        |
     *0 12    67  9      (17)
     *   |    |
     * start  end
     */
    UBOOT_DEBUG("DTBO Overlay JSON file token number get: %d\n", r);
    UBOOT_DEBUG("Loop start to process DTBO Overlay format \"Key:Value\" : \"ValueX\"\n");

    /* Json to map DTBO file*/
    for (i = 1; i < r; i++) {
        memset(separate_key, '\0', sizeof(separate_key));
        memset(separate_value, '\0', sizeof(separate_value));
        memset(ec_value, '\0', sizeof(ec_value));
        memset(filepath, '\0', sizeof(filepath));
        /* Separate Key:Value of dtbOverlay.json with ':'*/
        //UBOOT_DEBUG("EC DTBO Overlay get \"Key:Value\" \"%.*s\" \n", t[i].end - t[i].start, &filebuf[0] + t[i].start);
        get_token_value(t[i], filebuf, tokbuf_key);
        separator_p = strstr(tokbuf_key, KEY_SEPARATOR);
        strncpy(separate_key, tokbuf_key, (separator_p - tokbuf_key));
        strcpy(separate_value, separator_p + SEPARATOR_OFFSET);
        //UBOOT_DEBUG("EC DTBO get separated key value pair \"%s\" \"%s\"\n", separate_key, separate_value);
        //UBOOT_DEBUG("EC DTBO processing Separated Pair ask EC use key \"%s\"\n", separate_key);
        if (get_ec_value(separate_key, ec_value, MAX_VALUE_LENGTH) < 0)
        {
            //UBOOT_DEBUG("EC DTBO processing Separated Pair cannot read %s from ec, \"%s\" skip\n", separate_key, ec_value);
            i = i + 1;
            continue;
        } else {
            //UBOOT_DEBUG("EC DTBO processing Separated Pair ACK ec_value \"%s\"\n", ec_value);
            if (strcmp(separate_value, ec_value) != 0){
                //UBOOT_DEBUG("EC DTBO processing Separated Pair current separated value does not exist in ec, compare %s to %s skip\n", separate_value, ec_value);
                i = i + 1;
                continue;
            } else {
                //UBOOT_DEBUG("EC DTBO processing Separated Pair current separated value map success in ec, compare %s to %s\n", separate_value, ec_value);
                get_token_value(t[i + 1], filebuf, tokbuf_value);
                UBOOT_DEBUG("EC DTBO get overlay file name %s.dtbo\n", tokbuf_value);
                UBOOT_DEBUG("EC DTBO do %s.dtbo overlay\n", tokbuf_value);
                strcpy(filepath, FS_VENDOR_DTBO_POSITION); // filepath=/tvconfig/bsp/dtbo
                strcat(filepath, FS_FOLDER_SUFFIX);       // filepath=/tvconfig/bsp/dtbo/
                strcat(filepath, separate_key);            //filepath=/tvconfig/bsp/dtbo/separate_key
                strcat(filepath, FS_FOLDER_SUFFIX);       //filepath=/tvconfig/bsp/dtbo/separate_key/
                strcat(filepath, tokbuf_value);
                strcat(filepath, DTBO_SUFFIX);
                char *key_buf_dtbo = (char *) malloc(MAX_KEY_LENGTH);
                char *value_buf_dtbo = (char *) malloc(MAX_KEY_LENGTH);
                strcpy(key_buf_dtbo, separate_key);
                strcat(key_buf_dtbo, "_dtbo");
                strcpy(value_buf_dtbo, tokbuf_value);
                if(map_get(&EC_MAP, key_buf_dtbo) == NULL) {
                    printf("EC DTBO overlay Record dtbo file: %s : %s success\n", key_buf_dtbo, value_buf_dtbo);
                    map_set(&EC_MAP, key_buf_dtbo, value_buf_dtbo);
                }
                if (__load_and_overlay_dtbo(ft_addr, filepath))
                {
                    //printf("Cannot overlay dtbo file: %s skip\n", filepath);
                    i = i + 1;
                    continue;
                }
                //printf("EC DTBO overlay dtbo file: %s success\n", filepath);
            }
        }
        i = i + 1;
    }
    //T6y LDM temp fixed filepath=/vendor/ld_variation/ldm_AAA_0043.dtbo
    strcpy(filepath, "/vendor/tvconfig/bsp/dtbo/ld_variation/ldm_AAA_0043.dtbo");
    if (__load_and_overlay_dtbo(ft_addr, filepath))
    {
        //printf("Cannot overlay dtbo file: %s skip\n", filepath);
        return CMD_RET_FAILURE;
    }
    //printf("EC DTBO overlay dtbo file: %s success\n", filepath);

    UBOOT_DEBUG("EC DTBO overlay dtbo file exit\n");
    free(filebuf);
    free(ec_value);
    UBOOT_DEBUG("EC DTBO JSON file token number processed: %d\n", i);
    UBOOT_TRACE("EC DTS Overlay OUT \n");
    return CMD_RET_SUCCESS;

}

int do_ec_uboot_init(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[]){

    int err;
    int ret = CMD_RET_USAGE;

    char * value = (char *) malloc(MAX_VALUE_LENGTH + 1);
    char * version = (char *) malloc(MAX_VERSION_LENGTH + 1);

    if( EC_READY == 1){
        printf("EC already init skip init again\n");
        return 0;
    } else {
        printf("EC init\n");
    }

    map_init(&EC_MAP);

    if ((err=parse_ec_version()) < 0){
    
        /* Exception Handling */
    
        if (err == -1){
            UBOOT_DEBUG("get_ec_version fail | Pleae check the version.txt!\n");
            return CMD_RET_FAILURE;
        }
    }

    if ((err=check_ec_filepath_if_ready() < 0)) {
    
        /* Exception Handling */

        if (err == -2){
            UBOOT_DEBUG("get_ec_filepath fail | Pleae check the ec_path boot arg!\n");
        }
    }

    if ((err=base_read_to_Map() < 0)) {
    
        /* Exception Handling */

        if (err == -3){
            UBOOT_DEBUG("base_read_to_Map fail | Pleae check EC base Json file in %s\n", EC_PATH);
            return CMD_RET_FAILURE;
        }
    }
    UBOOT_DEBUG("base_read_to_Map success | Pleae check next\n");

    if ((err=loop_all_optional_ec() < 0)) {
        /* Exception Handling */
        if (err == CMD_RET_FAILURE){
            UBOOT_DEBUG("loop_all_optional_ec 0| Pleae check EC optional Json file in %s\n", EC_PATH);
            return CMD_RET_FAILURE;
        }
        if (err == -4){
            UBOOT_DEBUG("loop_all_optional_ec 1| Pleae check EC optional Json file in %s\n", EC_PATH);
            return CMD_RET_FAILURE;
        }
    }
    EC_READY = 1;

    /***************************************************************************
     *Expanding Json format 1
     *init your Json struct here.
     *Only support those files depoly in ext4 partition
        map_init(&YOUR_JSONMAP);
        ExBaseReadToMap(&YOUR_JSONMAP, YOUR_JSON_FILE_abPATH);
     *How to get absolute path?
     *console# pwd
     ***************************************************************************/
    map_init(&SOPQ_FlashPQ);
    err=ExBaseReadToMap(&SOPQ_FlashPQ, SOPQ_FlashPQ_MapJsonFilePath);
        /* Exception Handling */
    if (err == -3) {
        UBOOT_DEBUG("base_read_to_Map SOPQ_FlashPQ fail | Pleae check SOPQ_FlashPQ base Json file in %s\n", SOPQ_FlashPQ_MapJsonFilePath);
        SOPQ_FlashPQ_INIT = -1;
    } else {
        SOPQ_FlashPQ_INIT = 1;
        UBOOT_DEBUG("base_read_to_Map SOPQ_FlashPQ success | Pleae check next\n");
    }

    map_init(&SOPQ_LDM);
    err=ExBaseReadToMap(&SOPQ_LDM, SOPQ_LDM_MapJsonFilePath);
        /* Exception Handling */
    if (err == -3) {
        UBOOT_DEBUG("base_read_to_Map SOPQ_LDM fail | Pleae check SOPQ_LDM base Json file in %s\n", SOPQ_LDM_MapJsonFilePath);
        SOPQ_LDM_INIT = -1;
    } else {
        SOPQ_LDM_INIT = 1;
        UBOOT_DEBUG("base_read_to_Map SOPQ_LDM success | Pleae check next\n");
    }
    /* init by Panel Size*/
    map_init(&SOPQ_SOPQD_043);
    err=ExBaseReadToMap(&SOPQ_SOPQD_043, SOPQ_SOPQD_MapJsonFilePath_043);
        /* Exception Handling */
    if (err == -3) {
        UBOOT_DEBUG("base_read_to_Map SOPQ_SOPQD fail | Pleae check SOPQ_SOPQD base Json file in %s\n", SOPQ_SOPQD_MapJsonFilePath_043);
        SOPQ_SOPQD_INIT = -1;
    } else {
        SOPQ_SOPQD_INIT = 1;
        UBOOT_DEBUG("base_read_to_Map SOPQ_SOPQD success | Pleae check next\n");
    }

    map_init(&SOPQ_SOPQD_050);
    err=ExBaseReadToMap(&SOPQ_SOPQD_050, SOPQ_SOPQD_MapJsonFilePath_050);
        /* Exception Handling */
    if (err == -3) {
        UBOOT_DEBUG("base_read_to_Map SOPQ_SOPQD fail | Pleae check SOPQ_SOPQD base Json file in %s\n", SOPQ_SOPQD_MapJsonFilePath_050);
        SOPQ_SOPQD_INIT = -1;
    } else {
        SOPQ_SOPQD_INIT = 1;
        UBOOT_DEBUG("base_read_to_Map SOPQ_SOPQD success | Pleae check next\n");
    }

    map_init(&SOPQ_SOPQD_055);
    err=ExBaseReadToMap(&SOPQ_SOPQD_055, SOPQ_SOPQD_MapJsonFilePath_055);
        /* Exception Handling */
    if (err == -3) {
        UBOOT_DEBUG("base_read_to_Map SOPQ_SOPQD fail | Pleae check SOPQ_SOPQD base Json file in %s\n", SOPQ_SOPQD_MapJsonFilePath_055);
        SOPQ_SOPQD_INIT = -1;
    } else {
        SOPQ_SOPQD_INIT = 1;
        UBOOT_DEBUG("base_read_to_Map SOPQ_SOPQD success | Pleae check next\n");
    }

    map_init(&SOPQ_SOPQD_065);
    err=ExBaseReadToMap(&SOPQ_SOPQD_065, SOPQ_SOPQD_MapJsonFilePath_065);
        /* Exception Handling */
    if (err == -3) {
        UBOOT_DEBUG("base_read_to_Map SOPQ_SOPQD fail | Pleae check SOPQ_SOPQD base Json file in %s\n", SOPQ_SOPQD_MapJsonFilePath_065);
        SOPQ_SOPQD_INIT = -1;
    } else {
        SOPQ_SOPQD_INIT = 1;
        UBOOT_DEBUG("base_read_to_Map SOPQ_SOPQD success | Pleae check next\n");
    }

    map_init(&SOPQ_SOPQD_075);
    err=ExBaseReadToMap(&SOPQ_SOPQD_075, SOPQ_SOPQD_MapJsonFilePath_075);
        /* Exception Handling */
    if (err == -3) {
        UBOOT_DEBUG("base_read_to_Map SOPQ_SOPQD fail | Pleae check SOPQ_SOPQD base Json file in %s\n", SOPQ_SOPQD_MapJsonFilePath_075);
        SOPQ_SOPQD_INIT = -1;
    } else {
        SOPQ_SOPQD_INIT = 1;
        UBOOT_DEBUG("base_read_to_Map SOPQ_SOPQD success | Pleae check next\n");
    }

    map_init(&SOPQ_SOPQD_085);
    err=ExBaseReadToMap(&SOPQ_SOPQD_085, SOPQ_SOPQD_MapJsonFilePath_085);
        /* Exception Handling */
    if (err == -3) {
        UBOOT_DEBUG("base_read_to_Map SOPQ_SOPQD fail | Pleae check SOPQ_SOPQD base Json file in %s\n", SOPQ_SOPQD_MapJsonFilePath_085);
        SOPQ_SOPQD_INIT = -1;
    } else {
        SOPQ_SOPQD_INIT = 1;
        UBOOT_DEBUG("base_read_to_Map SOPQ_SOPQD success | Pleae check next\n");
    }

    map_init(&EC_VCOM);
    if ((err=ExBaseReadToMap(&EC_VCOM, EC_VCOM_PERSIST_FilePath) < 0)) {
        UBOOT_DEBUG("Can not find vcom.json in persist partition\n");
        err=ExBaseReadToMap(&EC_VCOM, EC_VCOM_CONFIG_FilePath);
            /* Exception Handling */
        if (err == -3){
            UBOOT_DEBUG("base_read_to_Map EC_VCOM fail | Pleae check EC_VCOM base Json file in %s\n", EC_VCOM_CONFIG_FilePath);
            EC_VCOM_INIT = -1;
        } else {
            EC_VCOM_INIT = 1;
            UBOOT_DEBUG("base_read_to_Map EC_VCOM success | Pleae check next\n");
        }
    } else {
		UBOOT_DEBUG("base_read_to_Map EC_VCOM success | Pleae check next\n");
		EC_VCOM_INIT = 1;
	}

    map_init(&SOPQ_DVCFG);
    err=ExBaseReadToMap(&SOPQ_DVCFG, SOPQ_DVCFG_MapJsonFilePath);
        /* Exception Handling */
    if (err == -3) {
        UBOOT_DEBUG("base_read_to_Map SOPQ_DVCFG fail | Pleae check SOPQ_DVCFG base Json file in %s\n", SOPQ_DVCFG_MapJsonFilePath);
        SOPQ_DVCFG_INIT = -1;
    } else {
        SOPQ_DVCFG_INIT = 1;
        UBOOT_DEBUG("base_read_to_Map SOPQ_DVCFG success | Pleae check next\n");
    }

    EX_EC_READY = 1;

    switch (argc){
        case 2:
            if(!strcmp(argv[1], "2")) {
                ret = get_ec_version(version, MAX_VERSION_LENGTH + 1);
                printf("EC version %s\n", version);
            } else {
                printf("Error input, try again\n");
            }
            break;
        case 3:
            if(!strcmp(argv[2], "version")) {
                ret = get_ec_version(version, MAX_VERSION_LENGTH + 1);
                printf("EC version %s\n", version);
                //memset(version, '\0', sizeof(version));
            } else {
                ret = get_ec_value(argv[2], value, MAX_VALUE_LENGTH);
                if (ret == 0) {
                    printf("EC info check Json Pair \"%s\": \"%s\"\n", argv[2], value);
                    //memset(value, '\0', sizeof(value));
                    //map_deinit(&EC_MAP);
                } else {
                    printf("EC info check Json Pair not found!! input again!!\n");
                }
            }
            break;
    }

    return ret;
}

U_BOOT_CMD(
    ec_uboot_init, 3, 1, do_ec_uboot_init,
    "EC init in Uboot.\n",
    NULL
);
