// SPDX-License-Identifier: (GPL-2.0 OR BSD-3-Clause)
/*
 * Copyright (c) 2021 MediaTek Inc.
 */

#ifndef _SONY_PANEL_HEADER
#define _SONY_PANEL_HEADER

#include <asm/gpio.h>

#define PANEL_ID_REG_FIRST  	0x00
#define PANEL_ID_REG_SECOND 	0xE0
#define PANEL_ID_LENGTH     	16
#define VCOM_DATA_REG_FIRST 	0x00
#define VCOM_DATA_REG_SECOND	0x10
#define VCOM_DATA_LENGTH    	8
#define VCOM_DATA_OFFSET    	4
#define VCOM_CRC_OFFSET     	6
#define VCOM_VALID_DATA_LENGTH	2
#define READ_TIMES          	10
#define REG_LENGTH           	2
#define PERSIST_PART         	"persist"
#define VCOM_FILE_PATH       	"/config/vcom.hex"
#define VCOM_FILE_PATH_UP       "/config/VCOM.hex"
#define PANEL_MISMATCH       	"panel_mismatch"
#define PANEL_ID             	"panel_id"

#define EC_DIMMING_CONFIG           "dimming_config"
#define EC_DAMP_CONFIG              "damp_config"
#define EC_MIC_CONFIG               "enable_builtin_mic"
#define EC_TUNER_CONFIG             "tuner_pin_config"
#define EC_LOCAL_DIMMING_VALUE      "local_dimming"
#define EC_ZERO_DIMMING_VALUE       "zero_dimming"
#define EC_DEMURA_FIXED_MODE        "demura_tconless_fixed_mode"
#define EC_DAMP1_VALUE              "damp1"
#define EC_DAMP1_TW_VALUE           "damp1_tw"
#define EC_DAMP2_SW_VALUE           "damp2_sw"
#define EC_DAMP2_TW_VALUE           "damp2_tw"
#define EC_TUNER_T2S2_VALUE         "t2s2"
#define EC_TUNER_ATSC3_VALUE        "atsc3"
#define EC_TUNER_PH_ISDB_VALUE      "ph_isdb"
#define EC_VCOM_JSON_TYPE_VALUE     "vcom_json_format_type"
#define EC_VCOM_DATA_VALUE          "Vcom_Data"
#define EC_VCOM_CRC_VALUE           "VcomCheckSum"
#define PMUX_LOCAL_DIMMING          "local_dimming"
#define PMUX_ZERO_DIMMING           "zero_dimming"
#define PMUX_DAMP                   "enable_damp"
#define PMUX_MIC                    "enable_mic"
#define PMUX_TUNER_T2S2             "tuner_t2s2"
#define PMUX_TUNER_ATSC3            "tuner_atsc3"
#define PMUX_TCONLESS               "tconless"

#define TCONLESS_PREFIX             "TCONLESS"
#define PANEL_ID_ERROR_KEY          "panel_id_err"
#define PANEL_ID_ERROR              "panel_id_err=1"
#define PANEL_ID_NO_ERROR           "panel_id_err=0"


struct sony_customer_priv {
	u32 i2c_addr;
	struct gpio_desc panel_less;
	struct gpio_desc tcon_less;
	struct gpio_desc panel_b_int;
	struct gpio_desc err_led;
	//struct gpio_desc trdy;
};

struct sony_customer_ops {
	void (*init_panel_id)(struct udevice *dev);
	void (*init_vcom_data)(struct udevice *dev);
	//void (*init_pmux)(struct udevice *dev);
	void (*init_pmux_cust)(struct udevice *dev, char *pinctrl_name);
	void (*enable_panel_rw)(struct udevice *dev, int value);
	//void (*enable_pmic)(struct udevice *dev, int value);
};

char* get_panel_id(void);
int get_panel_index(void);
bool is_sony_board(void);
char* get_vcom_data_from_eeprom(void);
char* get_vcom_data_from_emmc(void);
int get_panelless_status(void);
int get_tconless_status(void);
int sony_i2c_write_panel_id(struct udevice *dev, char* panel_id);
int sony_i2c_write_vcom_data(struct udevice *dev, long vcom_data);
void customer_start_time(void);
unsigned long customer_end_time(void);
unsigned long customer_panel_vcc_time(void);


#endif /* _SONY_PANEL_HEADER */
